@@ -20,9 +20,12 @@
 #include <stdlib.h>
 #include <stdint.h>
 #include <string.h>
-#include <math.h>
+
+#include <libavutil/avutil.h>
+#include <libavutil/pixfmt.h>
 
 #include "unpaper.h"
+#include "tools.h"
 
 /****************************************************************************
  * tool functions                                                           *
@@ -30,230 +33,153 @@
  *   - tool functions for image handling                                    *
  ****************************************************************************/
 
-
-/* --- arithmetic tool functions ------------------------------------------ */
-
-/**
- * Returns the quadratic square of a number.
- */
-double sqr(double d) {
-    return d*d;
+static inline uint8_t pixelGrayscale(uint8_t r, uint8_t g, uint8_t b) {
+    return (r + g + b) / 3;
 }
 
-
-/**
- * Converts degrees to radians.
- */
-double degreesToRadians(double d) {
-    return d * M_PI / 180.0;
-}
-
-
-/**
- * Converts radians to degrees.
- */
-double radiansToDegrees(double r) {
-    return r * 180.0 / M_PI;
-}
-
-
-/**
- * Limits an integer value to a maximum.
- */
-void limit(int* i, int max) {
-    if (*i > max) {
-        *i = max;
-    }
-}
-
-
 /* --- tool functions for image handling ---------------------------------- */
 
+static void getPixelComponents(AVFrame *image, int x, int y, uint8_t *r, uint8_t *g, uint8_t *b, uint8_t defval) {
+    uint8_t *pix;
+
+    if ( (x < 0) || (x >= image->width) || (y < 0) || (y >= image->height) ) {
+        *r = *g = *b = defval;
+        return;
+    }
+
+    switch(image->format) {
+    case AV_PIX_FMT_GRAY8:
+        pix = image->data[0] + (y * image->linesize[0] + x);
+        *r = *g = *b = *pix;
+        break;
+    case AV_PIX_FMT_Y400A:
+        pix = image->data[0] + (y * image->linesize[0] + x *2);
+        *r = *g = *b = *pix;
+        break;
+    case AV_PIX_FMT_RGB24:
+        pix = image->data[0] + (y * image->linesize[0] + x * 3);
+        *r = pix[0];
+        *g = pix[1];
+        *b = pix[2];
+        break;
+    case AV_PIX_FMT_MONOWHITE:
+        pix = image->data[0] + (y * image->linesize[0] + x / 8);
+        if ( *pix & (128 >> (x % 8)) )
+            *r = *g = *b = BLACK;
+        else
+            *r = *g = *b = WHITE;
+        break;
+    case AV_PIX_FMT_MONOBLACK:
+        pix = image->data[0] + (y * image->linesize[0] + x / 8);
+        if ( *pix & (128 >> (x % 8)) )
+            *r = *g = *b = WHITE;
+        else
+            *r = *g = *b = BLACK;
+        break;
+    default:
+        errOutput("unknown pixel format.");
+    }
+}
+
 /**
  * Allocates a memory block for storing image data and fills the IMAGE-struct
  * with the specified values.
  */
-void initImage(struct IMAGE* image, int width, int height, int bitdepth, bool color, int background) {
-    int size;
-    
-    size = width * height;
-    if ( color ) {
-        image->bufferGrayscale = (uint8_t*)malloc(size);
-        image->bufferLightness = (uint8_t*)malloc(size);
-        image->bufferDarknessInverse = (uint8_t*)malloc(size);
-        memset(image->bufferGrayscale, background, size);
-        memset(image->bufferLightness, background, size);
-        memset(image->bufferDarknessInverse, background, size);
-        size *= 3;
-    }
-    image->buffer = (uint8_t*)malloc(size);
-    memset(image->buffer, background, size);
-    if ( ! color ) {
-        image->bufferGrayscale = image->buffer;
-        image->bufferLightness = image->buffer;
-        image->bufferDarknessInverse = image->buffer;
-    }
-    image->width = width;
-    image->height = height;
-    image->bitdepth = bitdepth;
-    image->color = color;
-    image->background = background;
-}
+void initImage(AVFrame **image, int width, int height, int pixel_format, bool fill) {
+    int ret;
 
+    (*image) = av_frame_alloc();
+    (*image)->width = width;
+    (*image)->height = height;
+    (*image)->format = pixel_format;
 
-/**
- * Frees an image.
- */
-void freeImage(struct IMAGE* image) {    
-    free(image->buffer);
-    if (image->color) {
-        free(image->bufferGrayscale);
-        free(image->bufferLightness);
-        free(image->bufferDarknessInverse);
+    ret = av_frame_get_buffer(*image, 8);
+    if (ret < 0) {
+        char errbuff[1024];
+        av_strerror(ret, errbuff, sizeof(errbuff));
+        errOutput("unable to allocate buffer: %s", errbuff);
+    }
+
+    if ( fill ) {
+        for (int y = 0; y < (*image)->height; y++) {
+            for (int x = 0; x < (*image)->width; x++) {
+                setPixel(sheetBackground, x, y, *image);
+            }
+        }
     }
 }
 
-
-/**
- * Replaces one image with another.
- */
-void replaceImage(struct IMAGE* image, struct IMAGE* newimage) {    
-    freeImage(image);
-    // pass-back new image
-    *image = *newimage; // copy whole struct
-}
-
-
 /**
  * Sets the color/grayscale value of a single pixel.
  *
  * @return true if the pixel has been changed, false if the original color was the one to set
- */ 
-bool setPixel(int pixel, int x, int y, struct IMAGE* image) {
-    uint8_t* p;
-    int w, h;
-    int pos;
-    bool result;
-    uint8_t r, g, b;
-    
-    w = image->width;
-    h = image->height;
-    if ( (x < 0) || (x >= w) || (y < 0) || (y >= h) ) {
-        return false; //nop
-    } else {
-        pos = (y * w) + x;
-        r = (pixel >> 16) & 0xff;
-        g = (pixel >> 8) & 0xff;
-        b = pixel & 0xff;
-        if ( ! image->color ) {
-            p = &image->buffer[pos];
-            if ((r == g) && (r == b)) { // optimization (avoid division by 3)
-                pixel = r;
-            } else {
-                pixel = pixelGrayscale(r, g, b); // convert to gray (will already be in most cases, but we can't be sure)
-            }
-            if (*p != (uint8_t)pixel) {
-                *p = (uint8_t)pixel;
-                return true;
-            } else {
-                return false;
-            }
-        } else { // color
-            result = false;
-            p = &image->buffer[pos*3];
-            if (*p != r) {
-                *p = r;
-                result = true;
-            }
-            p++;
-            if (*p != g) {
-                *p = g;
-                result = true;
-            }
-            p++;
-            if (*p != b) {
-                *p = b;
-                result = true;
-            }
-            if ( result ) { // modified: update cached grayscale, lightness and brightnessInverse values
-                image->bufferGrayscale[pos] = pixelGrayscale(r, g, b);
-                image->bufferLightness[pos] = pixelLightness(r, g, b);
-                image->bufferDarknessInverse[pos] = pixelDarknessInverse(r, g, b);
-            }
-            return result;
-        }
-    }
-}
+ */
+bool setPixel(int pixel, int x, int y, AVFrame *image) {
+    uint8_t *pix;
 
+    if ( (x < 0) || (x >= image->width) || (y < 0) || (y >= image->height) ) {
+        return false; //nop
+    }
+
+    uint8_t r = (pixel >> 16) & 0xff;
+    uint8_t g = (pixel >> 8) & 0xff;
+    uint8_t b = pixel & 0xff;
+
+    uint8_t pixelbw = pixelGrayscale(r, g, b) < absBlackThreshold ? BLACK : WHITE;
+
+    switch(image->format) {
+    case AV_PIX_FMT_GRAY8:
+        pix = image->data[0] + (y * image->linesize[0] + x);
+        *pix = pixelGrayscale(r, g, b);
+        break;
+    case AV_PIX_FMT_Y400A:
+        pix = image->data[0] + (y * image->linesize[0] + x * 2);
+        pix[0] = pixelGrayscale(r, g, b);
+        pix[1] = 0xFF; // no alpha.
+        break;
+    case AV_PIX_FMT_RGB24:
+        pix = image->data[0] + (y * image->linesize[0] + x * 3);
+        pix[0] = r;
+        pix[1] = g;
+        pix[2] = b;
+        break;
+    case AV_PIX_FMT_MONOWHITE:
+        pixelbw = ~pixelbw; // reverse compared to following case
+    case AV_PIX_FMT_MONOBLACK:
+        pix = image->data[0] + (y * image->linesize[0] + x / 8);
+        if ( pixelbw == WHITE ) {
+            *pix = *pix | (128 >> (x % 8));
+        } else if ( pixelbw == BLACK ) {
+            *pix = *pix & ~(128 >> (x % 8));
+        }
+        break;
+    default:
+        errOutput("unknown pixel format.");
+    }
+    return true;
+}
 
 /**
  * Returns the color or grayscale value of a single pixel.
  * Always returns a color-compatible value (which may be interpreted as 8-bit grayscale)
  *
  * @return color or grayscale-value of the requested pixel, or WHITE if the coordinates are outside the image
- */ 
-int getPixel(int x, int y, struct IMAGE* image) {
-    const int w = image->width;
-    const int h = image->height;
-    if ( (x < 0) || (x >= w) || (y < 0) || (y >= h) ) {
-        return pixelValue(WHITE, WHITE, WHITE);
-    } else {
-        int pos = (y * w) + x;
-        if ( ! image->color ) {
-            const int pix = (uint8_t)image->buffer[pos];
-            return pixelValue(pix, pix, pix);
-        } else { // color
-            pos *= 3;
-            const uint8_t r = (uint8_t)image->buffer[pos++];
-            const uint8_t g = (uint8_t)image->buffer[pos++];
-            const uint8_t b = (uint8_t)image->buffer[pos];
-            return pixelValue(r, g, b);
-        }
-    }
+ */
+int getPixel(int x, int y, AVFrame *image) {
+    uint8_t r, g, b;
+    getPixelComponents(image, x, y, &r, &g, &b, WHITE);
+    return pixelValue(r, g, b);
 }
 
-
-/**
- * Returns a color component of a single pixel (0-255).
- *
- * @param colorComponent either RED, GREEN or BLUE
- * @return color or grayscale-value of the requested pixel, or WHITE if the coordinates are outside the image
- */ 
- /* (currently not used)
-int getPixelComponent(int x, int y, int colorComponent, struct IMAGE* image) {
-    int w, h;
-    int pos;
-
-    w = image->width;
-    h = image->height;
-    if ( (x < 0) || (x >= w) || (y < 0) || (y >= h) ) {
-        return WHITE;
-    } else {
-        pos = (y * w) + x;
-        if ( ! image->color ) {
-            return (uint8_t)image->buffer[pos];
-        } else { // color
-            return (uint8_t)image->buffer[(pos * 3) + colorComponent];
-        }
-    }
-}
-*/
-
 /**
  * Returns the grayscale (=brightness) value of a single pixel.
  *
  * @return grayscale-value of the requested pixel, or WHITE if the coordinates are outside the image
- */ 
-int getPixelGrayscale(int x, int y, struct IMAGE* image) {
-    const int w = image->width;
-    const int h = image->height;
-    const int pos = (y * w) + x;
-
-    if ( (x < 0) || (x >= w) || (y < 0) || (y >= h) ) {
-        return WHITE;
-    } else {
-        return image->bufferGrayscale[pos];
-    }
+ */
+static uint8_t getPixelGrayscale(int x, int y, AVFrame *image) {
+    uint8_t r, g, b;
+    getPixelComponents(image, x, y, &r, &g, &b, WHITE);
+    return pixelGrayscale(r, g, b);
 }
 
 
@@ -267,17 +193,11 @@ int getPixelGrayscale(int x, int y, struct IMAGE* image) {
  * For grayscale images, this value is equal to the pixel brightness.
  *
  * @return lightness-value (the higher, the lighter) of the requested pixel, or WHITE if the coordinates are outside the image
- */ 
-static int getPixelLightness(int x, int y, struct IMAGE* image) {
-    const int w = image->width;
-    const int h = image->height;
-    const int pos = (y * w) + x;
-
-    if ( (x < 0) || (x >= w) || (y < 0) || (y >= h) ) {
-        return WHITE;
-    } else {
-        return image->bufferLightness[pos];
-    }
+ */
+static uint8_t getPixelLightness(int x, int y, AVFrame *image) {
+    uint8_t r, g, b;
+    getPixelComponents(image, x, y, &r, &g, &b, WHITE);
+    return min3(r, g, b);
 }
 
 
@@ -291,72 +211,20 @@ static int getPixelLightness(int x, int y, struct IMAGE* image) {
  * For grayscale images, this value is equal to the pixel brightness.
  *
  * @return inverse-darkness-value (the LOWER, the darker) of the requested pixel, or WHITE if the coordinates are outside the image
- */ 
-int getPixelDarknessInverse(int x, int y, struct IMAGE* image) {
-    const int w = image->width;
-    const int h = image->height;
-    const int pos = (y * w) + x;
-
-    if ( (x < 0) || (x >= w) || (y < 0) || (y >= h) ) {
-        return WHITE;
-    } else {
-        return image->bufferDarknessInverse[pos];
-    }
+ */
+uint8_t getPixelDarknessInverse(int x, int y, AVFrame *image) {
+    uint8_t r, g, b;
+    getPixelComponents(image, x, y, &r, &g, &b, WHITE);
+    return max3(r, g, b);
 }
 
-
-/**
- * Sets the color/grayscale value of a single pixel to either black or white.
- *
- * @return true if the pixel has been changed, false if the original color was the one to set
- */ 
-static bool setPixelBW(int x, int y, struct IMAGE* image, int blackwhite) {
-    const int w = image->width;
-    const int h = image->height;
-    const int pos = (y * w) + x;
-
-    if ( (x < 0) || (x >= w) || (y < 0) || (y >= h) ) {
-        return false; //nop
-    } else {
-        if ( ! image->color ) {
-            uint8_t *p = &image->buffer[pos];
-            if (*p != blackwhite) {
-                *p = blackwhite;
-                return true;
-            } else {
-                return false;
-            }
-        } else { // color
-            uint8_t *p = &image->buffer[pos * 3];
-            bool result = false;
-
-            if (*p != blackwhite) {
-                *p = blackwhite;
-                result = true;
-            }
-            p++;
-            if (*p != blackwhite) {
-                *p = blackwhite;
-                result = true;
-            }
-            p++;
-            if (*p != blackwhite) {
-                *p = blackwhite;
-                result = true;
-            }
-            return result;
-        }
-    }
-}
-
-
 /**
  * Sets the color/grayscale value of a single pixel to white.
  *
  * @return true if the pixel has been changed, false if the original color was the one to set
- */ 
-static bool clearPixel(int x, int y, struct IMAGE* image) {
-    return setPixelBW(x, y, image, WHITE);
+ */
+static bool clearPixel(int x, int y, AVFrame *image) {
+    return setPixel(WHITE24, x, y, image);
 }
 
 
@@ -364,14 +232,12 @@ static bool clearPixel(int x, int y, struct IMAGE* image) {
  * Clears a rectangular area of pixels with either black or white.
  * @return The number of pixels actually changed from black (dark) to white.
  */
-int clearRect(int left, int top, int right, int bottom, struct IMAGE* image, int blackwhite) {
-    int x;
-    int y;
+int clearRect(const int left, const int top, const int right, const int bottom, AVFrame *image, const int blackwhite) {
     int count = 0;
 
-    for (y = top; y <= bottom; y++) {
-        for (x = left; x <= right; x++) {
-            if (setPixelBW(x, y, image, blackwhite)) {
+    for (int y = top; y <= bottom; y++) {
+        for (int x = left; x <= right; x++) {
+            if (setPixel(blackwhite, x, y, image)) {
                 count++;
             }
         }
@@ -383,37 +249,26 @@ int clearRect(int left, int top, int right, int bottom, struct IMAGE* image, int
 /**
  * Copies one area of an image into another.
  */
-void copyImageArea(int x, int y, int width, int height, struct IMAGE* source, int toX, int toY, struct IMAGE* target) {
-    int row;
-    int col;
+void copyImageArea(const int x, const int y, const int width, const int height, AVFrame *source, const int toX, const int toY, AVFrame *target) {
 
     // naive but generic implementation
-    for (row = 0; row < height; row++) {
-        for (col = 0; col < width; col++) {
+    for (int row = 0; row < height; row++) {
+        for (int col = 0; col < width; col++) {
             const int pixel = getPixel(x+col, y+row, source);
             setPixel(pixel, toX+col, toY+row, target);
         }
     }
 }
 
-
-/**
- * Copies a whole image into another.
- */
-void copyImage(struct IMAGE* source, int toX, int toY, struct IMAGE* target) {
-    copyImageArea(0, 0, source->width, source->height, source, toX, toY, target);
-}
-
-
 /**
  * Centers one area of an image inside an area of another image.
  * If the source area is smaller than the target area, is is equally
  * surrounded by a white border, if it is bigger, it gets equally cropped
  * at the edges.
  */
-static void centerImageArea(int x, int y, int w, int h, struct IMAGE* source, int toX, int toY, int ww, int hh, struct IMAGE* target) {
+static void centerImageArea(int x, int y, int w, int h, AVFrame *source, int toX, int toY, int ww, int hh, AVFrame *target) {
     if ((w < ww) || (h < hh)) { // white rest-border will remain, so clear first
-        clearRect(toX, toY, toX + ww - 1, toY + hh - 1, target, target->background);
+        clearRect(toX, toY, toX + ww - 1, toY + hh - 1, target, sheetBackground);
     }
     if (w < ww) {
         toX += (ww - w) / 2;
@@ -436,7 +291,7 @@ static void centerImageArea(int x, int y, int w, int h, struct IMAGE* source, in
 /**
  * Centers a whole image inside an area of another image.
  */
-void centerImage(struct IMAGE* source, int toX, int toY, int ww, int hh, struct IMAGE* target) {
+void centerImage(AVFrame *source, int toX, int toY, int ww, int hh, AVFrame *target) {
     centerImageArea(0, 0, source->width, source->height, source, toX, toY, ww, hh, target);
 }
 
@@ -444,57 +299,48 @@ void centerImage(struct IMAGE* source, int toX, int toY, int ww, int hh, struct
 /**
  * Returns the average brightness of a rectagular area.
  */
-int brightnessRect(int x1, int y1, int x2, int y2, struct IMAGE* image) {
-    int x;
-    int y;
-    int total = 0;
+uint8_t inverseBrightnessRect(const int x1, const int y1, const int x2, const int y2, AVFrame *image) {
+    unsigned int total = 0;
     const int count = (x2-x1+1)*(y2-y1+1);
 
-    for (x = x1; x <= x2; x++) {
-        for (y = y1; y <= y2; y++) {
-            const int pixel = getPixelGrayscale(x, y, image);
-            total += pixel;
+    for (int y = y1; y <= y2; y++) {
+        for (int x = x1; x <= x2; x++) {
+            total += getPixelGrayscale(x, y, image);
         }
     }
-    return total / count;
+    return WHITE - (total / count);
 }
 
 
 /**
- * Returns the average lightness of a rectagular area.
+ * Returns the inverseaverage lightness of a rectagular area.
  */
-int lightnessRect(int x1, int y1, int x2, int y2, struct IMAGE* image) {
-    int x;
-    int y;
-    int total = 0;
+uint8_t inverseLightnessRect(const int x1, const int y1, const int x2, const int y2, AVFrame *image) {
+    unsigned int total = 0;
     const int count = (x2-x1+1)*(y2-y1+1);
 
-    for (x = x1; x <= x2; x++) {
-        for (y = y1; y <= y2; y++) {
-            const int pixel = getPixelLightness(x, y, image);
-            total += pixel;
+    for (int y = y1; y <= y2; y++) {
+        for (int x = x1; x <= x2; x++) {
+            total += getPixelLightness(x, y, image);
         }
     }
-    return total / count;
+    return WHITE - (total / count);
 }
 
 
 /**
  * Returns the average darkness of a rectagular area.
  */
-int darknessInverseRect(int x1, int y1, int x2, int y2, struct IMAGE* image) {
-    int x;
-    int y;
-    int total = 0;
+uint8_t darknessRect(const int x1, const int y1, const int x2, const int y2, AVFrame *image) {
+    unsigned int total = 0;
     const int count = (x2-x1+1)*(y2-y1+1);
 
-    for (x = x1; x <= x2; x++) {
-        for (y = y1; y <= y2; y++) {
-            const int pixel = getPixelDarknessInverse(x, y, image);
-            total += pixel;
+    for (int y = y1; y <= y2; y++) {
+        for (int x = x1; x <= x2; x++) {
+            total += getPixelDarknessInverse(x, y, image);
         }
     }
-    return total / count;
+    return WHITE - (total / count);
 }
 
 
@@ -503,13 +349,11 @@ int darknessInverseRect(int x1, int y1, int x2, int y2, struct IMAGE* image) {
  * values ranges between minColor and maxBrightness. Optionally, the area can get
  * cleared with white color while counting.
  */
-int countPixelsRect(int left, int top, int right, int bottom, int minColor, int maxBrightness, bool clear, struct IMAGE* image) {
-    int x;
-    int y;
+int countPixelsRect(int left, int top, int right, int bottom, int minColor, int maxBrightness, bool clear, AVFrame *image) {
     int count = 0;
-    
-    for (y = top; y <= bottom; y++) {
-        for (x = left; x <= right; x++) {
+
+    for (int y = top; y <= bottom; y++) {
+        for (int x = left; x <= right; x++) {
             const int pixel = getPixelGrayscale(x, y, image);
             if ((pixel>=minColor) && (pixel <= maxBrightness)) {
                 if (clear==true) {
@@ -529,22 +373,19 @@ int countPixelsRect(int left, int top, int right, int bottom, int minColor, int
  * of 9 pixels if level==1, 16 pixels if level==2 and so on).
  * Optionally, the pixels can get cleared after counting.
  */
-static int countPixelNeighborsLevel(int x, int y, bool clear, int level, int whiteMin, struct IMAGE* image) {
-    int xx;
-    int yy;
+static int countPixelNeighborsLevel(int x, int y, bool clear, int level, int whiteMin, AVFrame *image) {
     int count = 0;
-    int pixel;
-    
+
     // upper and lower rows
-    for (xx = x - level; xx <= x + level; xx++) {
+    for (int xx = x - level; xx <= x + level; xx++) {
         // upper row
-        pixel = getPixelLightness(xx, y - level, image);
+        uint8_t pixel = getPixelLightness(xx, y - level, image);
         if (pixel < whiteMin) { // non-light pixel found
             if (clear == true) {
                 clearPixel(xx, y - level, image);
             }
             count++;
-        }        
+        }
         // lower row
         pixel = getPixelLightness(xx, y + level, image);
         if (pixel < whiteMin) {
@@ -552,18 +393,18 @@ static int countPixelNeighborsLevel(int x, int y, bool clear, int level, int whi
                 clearPixel(xx, y + level, image);
             }
             count++;
-        }        
+        }
     }
     // middle rows
-    for (yy = y-(level-1); yy <= y+(level-1); yy++) {
+    for (int yy = y-(level-1); yy <= y+(level-1); yy++) {
         // first col
-        pixel = getPixelLightness(x - level, yy, image);
+        uint8_t pixel = getPixelLightness(x - level, yy, image);
         if (pixel < whiteMin) {
             if (clear == true) {
                 clearPixel(x - level, yy, image);
             }
             count++;
-        }        
+        }
         // last col
         pixel = getPixelLightness(x + level, yy, image);
         if (pixel < whiteMin) {
@@ -571,7 +412,7 @@ static int countPixelNeighborsLevel(int x, int y, bool clear, int level, int whi
                 clearPixel(x + level, yy, image);
             }
             count++;
-        }        
+        }
     }
     /* old version, not optimized:
     for (yy = y-level; yy <= y+level; yy++) {
@@ -585,7 +426,7 @@ static int countPixelNeighborsLevel(int x, int y, bool clear, int level, int whi
                     count++;
                 }
             }
-        }    
+        }
     }*/
     return count;
 }
@@ -596,12 +437,12 @@ static int countPixelNeighborsLevel(int x, int y, bool clear, int level, int whi
  * reachable from the dark pixel at (x,y), without having to cross bright
  * pixels.
  */
-int countPixelNeighbors(int x, int y, int intensity, int whiteMin, struct IMAGE* image) {
-    int level;
+int countPixelNeighbors(int x, int y, int intensity, int whiteMin, AVFrame *image) {
     int count = 1; // assume self as set
     int lCount = -1;
-    
-    for (level = 1; (lCount != 0) && (level <= intensity); level++) { // can finish when one level is completely zero
+
+    // can finish when one level is completely zero
+    for (int level = 1; (lCount != 0) && (level <= intensity); level++) {
         lCount = countPixelNeighborsLevel(x, y, false, level, whiteMin, image);
         count += lCount;
     }
@@ -614,25 +455,17 @@ int countPixelNeighbors(int x, int y, int intensity, int whiteMin, struct IMAGE*
  * (x,y). This should be called only if it has previously been detected that
  * the amount of pixels to clear will be reasonable small.
  */
-void clearPixelNeighbors(int x, int y, int whiteMin, struct IMAGE* image) {
-    int level;
+void clearPixelNeighbors(int x, int y, int whiteMin, AVFrame *image) {
     int lCount = -1;
 
-    clearPixel(x, y, image);    
+    clearPixel(x, y, image);
 
-    for (level = 1; lCount != 0; level++) { // lCount will become 0, otherwise countPixelNeighbors() would previously have delivered a bigger value (and this here would not have been called)
+    // lCount will become 0, otherwise countPixelNeighbors() would previously have delivered a bigger value (and this here would not have been called)
+    for (int level = 1; lCount != 0; level++) {
         lCount = countPixelNeighborsLevel(x, y, true, level, whiteMin, image);
     }
 }
 
-
-/**
- * Flood-fill an area of pixels.
- * (Declaration of header for indirect recursive calls.)
- */
-void floodFill(int x, int y, int color, int maskMin, int maskMax, int intensity, struct IMAGE* image);
-
-
 /**
  * Solidly fills a line of pixels heading towards a specified direction
  * until color-changes in the pixels to overwrite exceed the 'intensity'
@@ -641,18 +474,16 @@ void floodFill(int x, int y, int color, int maskMin, int maskMax, int intensity,
  * @param stepX either -1 or 1, if stepY is 0, else 0
  * @param stepY either -1 or 1, if stepX is 0, else 0
  */
-static int fillLine(int x, int y, int stepX, int stepY, int color, int maskMin, int maskMax, int intensity, struct IMAGE* image) {
+static int fillLine(int x, int y, int stepX, int stepY, int color, uint8_t maskMin, uint8_t maskMax, int intensity, AVFrame *image) {
     int distance = 0;
     int intensityCount = 1; // first pixel must match, otherwise directly exit
     const int w = image->width;
     const int h = image->height;
 
     while (true) {
-        int pixel;
-
         x += stepX;
         y += stepY;
-        pixel = getPixelGrayscale(x, y, image);
+        uint8_t pixel = getPixelGrayscale(x, y, image);
         if ((pixel>=maskMin) && (pixel<=maskMax)) {
             intensityCount = intensity; // reset counter
         } else {
@@ -677,10 +508,8 @@ static int fillLine(int x, int y, int stepX, int stepY, int color, int maskMin,
  * @param stepY either -1 or 1, if stepX is 0, else 0
  * @see fillLine()
  */
-static void floodFillAroundLine(int x, int y, int stepX, int stepY, int distance, int color, int maskMin, int maskMax, int intensity, struct IMAGE* image) {
-    int d;
-    
-    for (d = 0; d < distance; d++) {
+static void floodFillAroundLine(int x, int y, int stepX, int stepY, int distance, int color, int maskMin, int maskMax, int intensity, AVFrame *image) {
+    for (int d = 0; d < distance; d++) {
         if (stepX != 0) {
             x += stepX;
             floodFill(x, y + 1, color, maskMin, maskMax, intensity, image); // indirect recursion
@@ -699,7 +528,7 @@ static void floodFillAroundLine(int x, int y, int stepX, int stepY, int distance
  *
  * @see earlier header-declaration to enable indirect recursive calls
  */
-void floodFill(int x, int y, int color, int maskMin, int maskMax, int intensity, struct IMAGE* image) {
+void floodFill(int x, int y, int color, int maskMin, int maskMax, int intensity, AVFrame *image) {
     // is current pixel to be filled?
     const int pixel = getPixelGrayscale(x, y, image);
     if ((pixel>=maskMin) && (pixel<=maskMax)) {
